from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Generic, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.models.lite.user import LiteUser
from mipac.types.invite import IInviteCode, IPartialInviteCode
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


T = TypeVar("T", bound=IPartialInviteCode)


class PartialInviteCode(Generic[T], AbstractModel):
    def __init__(self, raw_invite_code: T, *, client: ClientManager) -> None:
        self._raw_invite_code: T = raw_invite_code
        self._client = client

    @property
    def code(self) -> str:
        return self._raw_invite_code["code"]


class InviteCode(PartialInviteCode[IInviteCode]):
    def __init__(self, raw_invite_code: IInviteCode, *, client: ClientManager) -> None:
        super().__init__(raw_invite_code=raw_invite_code, client=client)

    @property
    def id(self) -> str:
        return self._raw_invite_code["id"]

    @property
    def expires_at(self) -> datetime | None:
        return (
            str_to_datetime(self._raw_invite_code["expires_at"])
            if self._raw_invite_code["expires_at"]
            else None
        )

    @property
    def created_at(self) -> datetime | None:
        return (
            str_to_datetime(self._raw_invite_code["created_at"])
            if self._raw_invite_code["created_at"]
            else None
        )

    @property
    def created_by(self) -> datetime | None:
        return (
            str_to_datetime(self._raw_invite_code["created_by"])
            if self._raw_invite_code["created_by"]
            else None
        )

    @property
    def used_by(self) -> LiteUser | None:
        return (
            LiteUser(self._raw_invite_code["used_by"], client=self._client)
            if self._raw_invite_code["used_by"]
            else None
        )

    @property
    def used_at(self) -> datetime | None:
        return (
            str_to_datetime(self._raw_invite_code["used_at"])
            if self._raw_invite_code["used_at"]
            else None
        )

    @property
    def used(self) -> bool:
        return self._raw_invite_code["used"]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.models.lite.channel import ChannelLite
from mipac.types.channel import IChannel

if TYPE_CHECKING:
    from mipac.manager import ClientManager


class Channel(ChannelLite[IChannel]):
    def __init__(self, channel: IChannel, *, client: ClientManager) -> None:
        super().__init__(channel=channel, client=client)

    @property
    def has_unread_note(self) -> bool:
        return self._channel["has_unread_note"]

    @property
    def is_following(self) -> bool | None:
        return self._channel.get("is_following")

    @property
    def is_favorited(self) -> bool | None:
        return self._channel.get("is_favorited")



from typing import TypedDict

from mipac.types.user import ILiteUser, IUserDetailed


class IFederationFollowCommon(TypedDict):
    id: str
    created_at: str
    followee_id: str
    followee: IUserDetailed
    follower_id: str
    follower: IUserDetailed


class IFederationFollower(IFederationFollowCommon):
    ...


class IFederationFollowing(IFederationFollowCommon):
    ...


class IFollowRequest(TypedDict):
    id: str
    follower: ILiteUser
    followee: ILiteUser



from __future__ import annotations

from typing import TYPE_CHECKING, TypedDict

if TYPE_CHECKING:
    from mipac.types.user import IUserDetailed


class IMuteUser(TypedDict):
    id: str
    created_at: str
    mutee_id: str
    mutee: IUserDetailed



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.antenna import Antenna
from mipac.models.note import Note
from mipac.types.antenna import IAntenna, IAntennaReceiveSource
from mipac.types.note import INote
from mipac.utils.format import remove_dict_empty
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class ClientAntennaActions(AbstractAction):
    def __init__(
        self, *, antenna_id: str | None = None, session: HTTPClient, client: ClientManager
    ):
        self._antenna_id: str | None = antenna_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def delete(self, antenna_id: str | None = None) -> bool:
        """
        Delete antenna from identifier

        Parameters
        ----------
        antenna_id : str | None, optional
            target identifier

        Raises
        ------
        ParameterError
            antenna id is required
        Returns
        -------
        bool
            success or failure
        """
        antenna_id = antenna_id or self._antenna_id
        if antenna_id is None:
            raise ParameterError("antenna id is required")

        body = {"antennaId": antenna_id}
        res: bool = await self._session.request(
            Route("POST", "/api/antennas/delete"), auth=True, json=body
        )
        return res

    async def show(self, antenna_id: str | None = None) -> Antenna:
        """Show antenna from identifier

        Parameters
        ----------
        antenna_id : str | None, optional
            target identifier, by default None

        Returns
        -------
        Antenna
            antenna object

        Raises
        ------
        ParameterError
            antenna id is required
        """
        antenna_id = antenna_id or self._antenna_id
        if antenna_id is None:
            raise ParameterError("antenna id is required")

        body = {"antennaId": antenna_id}
        res_antenna: IAntenna = await self._session.request(
            Route("POST", "/api/antennas/show"), auth=True, json=body
        )
        return Antenna(res_antenna, client=self._client)

    async def get_notes(
        self,
        antenna_id: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: str | None = None,
        until_date: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[Note, None]:
        antenna_id = antenna_id or self._antenna_id
        if antenna_id is None:
            raise ParameterError("antenna id is required")

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = remove_dict_empty(
            {
                "antennaId": antenna_id,
                "limit": limit,
                "sinceId": since_id,
                "untilId": until_id,
                "sinceDate": since_date,
                "untilDate": until_date,
            }
        )

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/antennas/notes"), json=body
        )

        while True:
            res_notes = await pagination.next()
            for res_note in res_notes:
                yield Note(res_note, client=self._client)

            if get_all is False or pagination.is_final:
                break

    async def update(
        self,
        name: str,
        src: IAntennaReceiveSource,
        keywords: list[list[str]],
        exclude_keywords: list[list[str]],
        users: list[str],
        case_sensitive: bool,
        with_replies: bool,
        with_file: bool,
        notify: bool,
        user_list_id: str | None = None,
        antenna_id: str | None = None,
    ) -> Antenna:
        """Update an antenna.

        Parameters
        ----------
        name : str
            Name of the antenna.
        src : IAntennaReceiveSource
            Receive source of the antenna.
        keywords : list[list[str]]
            Receive keywords.
        exclude_keywords : list[list[str]]
            Excluded keywords.
        users : list[str]
            List of target user ID. Required when selecting 'users' as the receive source.
        case_sensitive : bool
            Whether to differentiate between uppercase and lowercase letters.
        with_replies : bool
            Whether to include replies.
        with_file : bool
            Whether to limit to notes with attached files.
        notify : bool
            Whether to notify for new notes.
        user_list_id : str | None, default None
            List of user IDs when selecting 'users' as the receive source for the antenna.

        Returns
        -------
        Antenna
            The created antenna.
        """

        antenna_id = antenna_id or self._antenna_id
        if antenna_id is None:
            raise ParameterError("antenna id is required")

        if (
            all(
                [
                    name,
                    src,
                    keywords,
                ]
            )
            is False
        ):
            raise ParameterError("Required parameters are missing")
        body = {
            "antennaId": antenna_id,
            "name": name,
            "src": src,
            "userListId": user_list_id,
            "keywords": keywords,
            "excludeKeywords": exclude_keywords,
            "users": users,
            "caseSensitive": case_sensitive,
            "withReplies": with_replies,
            "withFile": with_file,
            "notify": notify,
        }

        res_antenna: IAntenna = await self._session.request(
            Route("POST", "/api/antennas/update"), auth=True, json=body, remove_none=False
        )

        return Antenna(res_antenna, client=self._client)


class AntennaActions(ClientAntennaActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def create(
        self,
        name: str,
        src: IAntennaReceiveSource,
        keywords: list[list[str]],
        exclude_keywords: list[list[str]] | None = None,
        users: list[str] | None = None,
        case_sensitive: bool = False,
        with_replies: bool = False,
        with_file: bool = False,
        notify: bool = False,
        user_list_id: str | None = None,
    ) -> Antenna:
        """Create an antenna.

        Parameters
        ----------
        name : str
            Name of the antenna.
        src : IAntennaReceiveSource
            Receive source of the antenna.
        keywords : list[list[str]]
            Receive keywords.
        exclude_keywords : list[list[str]] | None, default None
            Excluded keywords.
        users : list[str] | None, default None
            List of target user ID. Required when selecting 'users' as the receive source.
        case_sensitive : bool, default False
            Whether to differentiate between uppercase and lowercase letters.
        with_replies : bool, default False
            Whether to include replies.
        with_file : bool, default False
            Whether to limit to notes with attached files.
        notify : bool, default False
            Whether to notify for new notes.
        user_list_id : str | None, default None
            List of user IDs when selecting 'users' as the receive source for the antenna.

        Returns
        -------
        Antenna
            The created antenna.
        """

        if users is None:
            users = [""]
        if exclude_keywords is None:
            exclude_keywords = [[]]

        if (
            all(
                [
                    name,
                    src,
                    keywords,
                ]
            )
            is False
        ):
            raise ParameterError("Required parameters are missing")
        body = {
            "name": name,
            "src": src,
            "userListId": user_list_id,
            "keywords": keywords,
            "excludeKeywords": exclude_keywords,
            "users": users,
            "caseSensitive": case_sensitive,
            "withReplies": with_replies,
            "withFile": with_file,
            "notify": notify,
        }

        res_antenna: IAntenna = await self._session.request(
            Route("POST", "/api/antennas/create"), auth=True, json=body, remove_none=False
        )

        return Antenna(res_antenna, client=self._client)

    async def get_list(self) -> list[Antenna]:
        res_antennas: list[IAntenna] = await self._session.request(
            Route("POST", "/api/antennas/list"), auth=True
        )

        return [Antenna(res_antenna, client=self._client) for res_antenna in res_antennas]



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.config import config
from mipac.errors.base import NotSupportVersion, NotSupportVersionText, ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.admin import IndexStat, ModerationLog, ServerInfo, UserIP
from mipac.models.meta import AdminMeta
from mipac.models.user import MeDetailed, UserDetailed
from mipac.types.admin import IIndexStat, IModerationLog, IServerInfo, ITableStats, IUserIP
from mipac.types.meta import IAdminMeta, IUpdateMetaBody
from mipac.types.user import IMeDetailed, IUserDetailed, is_me_detailed
from mipac.utils.cache import cache
from mipac.utils.format import convert_dict_keys_to_camel
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class AdminActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    async def get_meta(self, detail: bool = False) -> AdminMeta:
        res: IAdminMeta = await self.__session.request(
            Route("POST", "/api/admin/meta"),
            json={"detail": detail},
            auth=True,
            lower=True,
        )
        return AdminMeta(res, client=self.__client)

    async def get_invite(self) -> bool:
        return bool(await self.__session.request(Route("POST", "/api/admin/invite")))

    async def vacuum(self, full: bool = False, analyze: bool = False) -> bool:
        body = {"full": full, "analyze": analyze}
        return bool(
            await self.__session.request(Route("POST", "/api/admin/vacuum"), auth=True, json=body)
        )

    async def update_user_note(self, user_id: str, text: str) -> bool:
        if config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)
        body = {"userId": user_id, "text": text}
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/update-user-note"), auth=True, json=body
            )
        )

    async def update_meta(self, meta: IUpdateMetaBody) -> bool:
        body = convert_dict_keys_to_camel(
            meta, replace_list={"tos_text_url": "ToSTextUrl", "tos_url": "ToSUrl"}
        )

        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/update-meta"),
                json=body,
                auth=True,
                lower=True,
                remove_none=False,
            )
        )

    async def unsuspend_user(self, user_id: str) -> bool:
        """Unsuspend user with specified Id

        Parameters
        ----------
        user_id : str
            Id of user to unsuspend

        Returns
        -------
        bool
            success or failed
        """

        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/unsuspend-user"), json={"userId": user_id}, auth=True
            )
        )

    async def unsilence_user(self, user_id: str) -> bool:
        """Unsilence user with specified Id

        Parameters
        ----------
        user_id : str
            Id of user to unsilence

        Returns
        -------
        bool
            success or failed
        """

        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/unsilence-user"), json={"userId": user_id}, auth=True
            )
        )

    async def suspend_user(self, user_id: str) -> bool:
        """Suspends the user for the specified Id

        Parameters
        ----------
        user_id : str
            Id of user to suspend

        Returns
        -------
        bool
            success or failed
        """
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/suspend-user"), json={"userId": user_id}, auth=True
            )
        )

    async def silence_user(self, user_id: str) -> bool:
        """Silences the user of the specified Id

        Parameters
        ----------
        user_id : str
            Id of user to silence

        Returns
        -------
        bool
            success or failed
        """
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/silence-user"), json={"userId": user_id}, auth=True
            )
        )

    async def get_moderation_logs(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[ModerationLog, None]:
        if config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = {"limit": limit, "sinceId": since_id, "untilId": until_id}
        pagination = Pagination[IModerationLog](
            self.__session, Route("POST", "/api/admin/show-moderation-logs"), json=body
        )

        while True:
            res_moderation_logs = await pagination.next()
            for res_moderation_log in res_moderation_logs:
                yield ModerationLog(res_moderation_log, client=self.__client)

    @cache("server_info")
    async def get_server_info(self, **kwargs) -> ServerInfo:
        server_info_payload: IServerInfo = await self.__session.request(
            Route("POST", "/api/admin/server-info"), auth=True, lower=True
        )
        return ServerInfo(server_info_payload)

    async def fetch_server_info(self) -> ServerInfo:
        return await self.get_server_info(cache_override=True)

    async def send_email(self, to: str, subject: str, text: str) -> bool:
        body = {"to": to, "subject": subject, "text": text}
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/send-email"), auth=True, json=body
            )
        )

    async def resolve_abuse_user_report(self, report_id: str, forward: bool = False) -> bool:
        if config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        body = {"reportId": report_id, "forward": forward}
        return bool(
            await self.__session.request(
                Route("POST", "/api/admin/resolve-abuse-user-report"), auth=True, json=body
            )
        )

    async def reset_password(self, user_id: str) -> str:
        """target user's password reset

        Parameters
        ----------
        user_id : str
            target user's id

        Returns
        -------
        str
            new password
        """
        return await self.__session.request(
            Route("POST", "/api/admin/reset-password"), auth=True, json={"userId": user_id}
        )

    async def get_table_stats(self) -> dict[str, ITableStats]:
        return await self.__session.request(Route("POST", "/api/admin/get-table-stats"), auth=True)

    async def get_index_stats(self) -> list[IndexStat]:
        res: list[IIndexStat] = await self.__session.request(
            Route("POST", "/api/admin/get-index-stats"), auth=True
        )
        return [IndexStat(i) for i in res]

    async def get_user_ips(self, user_id: str) -> list[UserIP]:
        if config.use_version < 12:
            raise NotSupportVersion(NotSupportVersionText)

        res: list[IUserIP] = await self.__session.request(
            Route("POST", "/api/admin/get-user-ips"),
            auth=True,
            json={"userId": user_id},
            lower=True,
        )
        return [UserIP(i) for i in res]

    async def show_user(self, user_id: str) -> UserDetailed:
        res: IUserDetailed = await self.__session.request(
            Route("POST", "/api/admin/show-user"), auth=True, json={"userId": user_id}
        )
        return UserDetailed(res, client=self.__client)

    async def show_users(
        self,
        limit: int = 10,
        offset: int = 0,
        sort: str | None = None,
        state: str = "all",
        origin: str = "combined",
        username: str | None = None,
        hostname: str | None = None,
    ) -> list[MeDetailed | UserDetailed]:
        body = {
            "limit": limit,
            "offset": offset,
            "sort": sort,
            "state": state,
            "origin": origin,
            "username": username,
            "hostname": hostname,
        }
        res: list[IMeDetailed | IUserDetailed] = await self.__session.request(
            Route("POST", "/api/admin/show-users"), auth=True, json=body
        )

        return [
            MeDetailed(i, client=self.__client)
            if is_me_detailed(i, config.account_id)
            else UserDetailed(i, client=self.__client)
            for i in res
        ]



from __future__ import annotations

from typing import TYPE_CHECKING, Any, AsyncGenerator, Literal

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText, ParameterError
from mipac.http import Route
from mipac.models.roles import Role, RoleUser
from mipac.types.meta import IPolicies
from mipac.types.roles import IRole, IRoleUser
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class AdminRoleModelActions(AbstractAction):
    def __init__(self, role_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client
        self._role_id: str | None = role_id

    async def update(
        self,
        name: str,
        description: str,
        role_id: str | None = None,
        color: str | None = None,
        iconUrl: str | None = None,
        target: Literal["manual", "conditional"] = "manual",
        cond_formula: dict[Any, Any] | None = None,
        is_public: bool = False,
        is_moderator: bool = False,
        is_administrator: bool = False,
        as_badge: bool = False,
        can_edit_members_by_moderator: bool = False,
        policies: dict[Any, Any] | None = None,
    ) -> bool:
        if self._client._config.use_version >= 13:
            role_id = self._role_id or role_id
            if role_id is None:
                raise ParameterError("role_idは必須です")
            body = {
                "roleId": role_id,
                "name": name,
                "description": description,
                "color": color,
                "iconUrl": iconUrl,
                "target": target,
                "condFormula": cond_formula or {},
                "isPublic": is_public,
                "isModerator": is_moderator,
                "isAdministrator": is_administrator,
                "asBadge": as_badge,
                "canEditMembersByModerator": can_edit_members_by_moderator,
                "policies": policies or {},
            }
            res: bool = await self._session.request(
                Route("POST", "/api/admin/roles/update"),
                json=body,
                auth=True,
                lower=True,
                remove_none=False,
            )
            return res
        raise NotSupportVersion(NotSupportVersionText)

    async def delete(self, role_id: str | None = None) -> bool:
        if self._client._config.use_version >= 13:
            role_id = self._role_id or role_id
            if role_id is None:
                raise ParameterError("role_idは必須です")
            res: bool = await self._session.request(
                Route("POST", "/api/admin/roles/delete"),
                auth=True,
                json={"roleId": role_id},
                lower=True,
            )
            return res
        raise NotSupportVersion(NotSupportVersionText)

    async def assign(
        self, user_id: str, role_id: str | None = None, expires_at: int | None = None
    ) -> bool:
        """指定したユーザーに指定したロールを付与します

        Parameters
        ----------
        role_id : str
            ロールのID
        user_id : str
            ロールを付与する対象のユーザーID
        expires_at : int | None, optional
            いつまでロールを付与するか, by default None

        Returns
        -------
        bool
            成功したか否か
        """
        if self._client._config.use_version >= 13:
            if role_id is None:
                raise ParameterError("role_idは必須です")
            body = {"roleId": role_id, "userId": user_id, "expiresAt": expires_at}
            res: bool = await self._session.request(
                Route("POST", "/api/admin/roles/assign"), auth=True, json=body
            )
            return res
        raise NotSupportVersion(NotSupportVersionText)

    async def unassign(self, user_id: str, role_id: str | None = None) -> bool:
        """指定したユーザーに指定したロールを付与します

        Parameters
        ----------
        role_id : str
            ロールのID
        user_id : str
            ロールを付与する対象のユーザーID
        expires_at : int | None, optional
            いつまでロールを付与するか, by default None

        Returns
        -------
        bool
            成功したか否か
        """
        if self._client._config.use_version >= 13:
            role_id = self._role_id or role_id
            if role_id is None:
                raise ParameterError("role_idは必須です")
            body = {"roleId": role_id, "userId": user_id}
            res: bool = await self._session.request(
                Route("POST", "/api/admin/roles/unassign"), auth=True, json=body
            )
            return res
        raise NotSupportVersion(NotSupportVersionText)

    async def show(self, role_id: str | None = None) -> Role:
        if self._client._config.use_version >= 13:
            role_id = self._role_id or role_id
            if role_id is None:
                raise ParameterError("role_idは必須です")
            res: IRole = await self._session.request(
                Route("POST", "/api/admin/roles/show"),
                json={"roleId": role_id},
                auth=True,
                lower=True,
            )
            return Role(res, client=self._client)
        raise NotSupportVersion(NotSupportVersionText)

    async def get_users(
        self,
        role_id: str | None = None,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 100,
        get_all: bool = False,
    ) -> AsyncGenerator[RoleUser, None]:
        role_id = self._role_id or role_id

        if self._client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        if role_id is None:
            raise ParameterError("role_idは必須です")

        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")

        if get_all:
            limit = 100

        body = {"limit": limit, "sinceId": since_id, "untilId": until_id, "roleId": role_id}

        pagination = Pagination[IRoleUser](
            self._session, Route("POST", "/api/admin/roles/users"), json=body
        )

        while True:
            raw_role_users = await pagination.next()
            for role_user in raw_role_users:
                yield RoleUser(role_user, client=self._client)

            if get_all is False or pagination.is_final:
                break


class AdminRoleActions(AdminRoleModelActions):
    def __init__(self, role_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        super().__init__(role_id=role_id, session=session, client=client)

    async def create(
        self,
        name: str,
        description: str,
        color: str | None = None,
        iconUrl: str | None = None,
        target: Literal["manual", "conditional"] = "manual",
        cond_formula: dict[Any, Any] | None = None,
        is_public: bool = False,
        is_moderator: bool = False,
        is_administrator: bool = False,
        as_badge: bool = False,
        can_edit_members_by_moderator: bool = False,
        policies: dict[Any, Any] | None = None,
        is_explorable: bool = False,
    ) -> Role:
        if self._client._config.use_version >= 13:
            body = {
                "name": name,
                "description": description,
                "color": color,
                "iconUrl": iconUrl,
                "target": target,
                "condFormula": cond_formula or {},
                "isPublic": is_public,
                "isModerator": is_moderator,
                "isAdministrator": is_administrator,
                "asBadge": as_badge,
                "canEditMembersByModerator": can_edit_members_by_moderator,
                "policies": policies or {},
                "isExplorable": is_explorable,
            }
            res: IRole = await self._session.request(
                Route("POST", "/api/admin/roles/create"),
                auth=True,
                json=body,
                lower=True,
                remove_none=False,
            )
            return Role(res, client=self._client)
        raise NotSupportVersion(NotSupportVersionText)

    async def get_list(self) -> list[Role]:
        if self._client._config.use_version >= 13:
            res: list[IRole] = await self._session.request(
                Route("POST", "/api/admin/roles/list"), auth=True, lower=True
            )
            return [Role(i, client=self._client) for i in res]
        raise NotSupportVersion(NotSupportVersionText)

    async def update_default_policies(self, policies: IPolicies):
        if self._client._config.use_version >= 13:
            body = {
                "policies": {
                    "gtlAvailable": policies.get("gtl_available"),
                    "ltlAvailable": policies.get("ltl_available"),
                    "canPublicNote": policies.get("can_public_note"),
                    "canInvite": policies.get("can_invite"),
                    "canManageCustomEmojis": policies.get("can_manage_custom_emojis"),
                    "canHideAds": policies.get("can_hide_ads"),
                    "driveCapacityMb": policies.get("drive_capacity_mb"),
                    "pinLimit": policies.get("pin_limit"),
                    "antennaLimit": policies.get("antenna_limit"),
                    "wordMuteLimit": policies.get("word_mute_limit"),
                    "webhookLimit": policies.get("webhook_limit"),
                    "clipLimit": policies.get("clip_limit"),
                    "noteEachClipsLimit": policies.get("note_each_clips_limit"),
                    "userListLimit": policies.get("user_list_limit"),
                    "userEachUserListsLimit": policies.get("user_each_user_lists_limit"),
                    "rateLimitFactor": policies.get("rate_limit_factor"),
                }
            }
            res = await self._session.request(
                Route("POST", "/api/admin/roles/update-default-policies"),
                auth=True,
                lower=True,
                json=body,
            )
            return res
        raise NotSupportVersion(NotSupportVersionText)
